"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DriverTests = void 0;
const query_orchestrator_1 = require("@cubejs-backend/query-orchestrator");
const shared_1 = require("@cubejs-backend/shared");
const crypto_1 = __importDefault(require("crypto"));
const dedent_1 = __importDefault(require("dedent"));
const stream_1 = require("stream");
const assert_1 = __importDefault(require("assert"));
class DriverTests {
    driver;
    options;
    constructor(driver, options = {}) {
        this.driver = driver;
        this.options = options;
    }
    release() {
        return this.driver.release();
    }
    static QUERY = `
    SELECT id, amount, status
    FROM (
      SELECT 1 AS id, 100 AS amount, 'new' AS status
      UNION ALL
      SELECT 2 AS id, 200 AS amount, 'new' AS status
      UNION ALL
      SELECT 3 AS id, 400 AS amount, 'processed' AS status
      UNION ALL
      SELECT 4 AS id, 500 AS amount, NULL AS status
    ) AS data
    ORDER BY 1
  `;
    static ROWS = [
        { id: 1, amount: 100, status: 'new' },
        { id: 2, amount: 200, status: 'new' },
        { id: 3, amount: 400, status: 'processed' },
        { id: 4, amount: 500, status: null },
    ];
    getExpectedCsvRows() {
        return (0, dedent_1.default) `
      orders__status,orders__amount
      new,300
      processed,400
      ,500
    `;
    }
    async testQuery() {
        const rows = await this.driver.query(DriverTests.QUERY, []);
        const expectedRows = this.options.expectStringFields ? DriverTests.rowsToString(DriverTests.ROWS) : DriverTests.ROWS;
        expect(rows).toEqual(expectedRows);
    }
    async testStream() {
        expect(this.driver.stream).toBeDefined();
        const tableData = await this.driver.stream(DriverTests.QUERY, [], { highWaterMark: 100 });
        expect(tableData.rowStream instanceof stream_1.Readable);
        const rows = await (0, shared_1.streamToArray)(tableData.rowStream);
        const expectedRows = this.options.expectStringFields ? DriverTests.rowsToString(DriverTests.ROWS) : DriverTests.ROWS;
        expect(rows).toEqual(expectedRows);
    }
    async testUnload() {
        const query = `
      SELECT orders.status AS orders__status, sum(orders.amount) AS orders__amount        
      FROM (${DriverTests.QUERY}) AS orders
      GROUP BY 1
      ORDER BY 2
    `;
        const tableName = await this.createUnloadTable(query);
        (0, assert_1.default)(this.driver.unload);
        const data = await this.driver.unload(tableName, { maxFileSize: 64 });
        expect(data.csvFile.length).toEqual(1);
        const string = await (0, shared_1.downloadAndGunzip)(data.csvFile[0]);
        let expectedCsvRows = this.getExpectedCsvRows();
        if (this.options.delimiter) {
            expectedCsvRows = expectedCsvRows.replaceAll(/,/g, this.options.delimiter);
        }
        const expectedRows = this.options.csvNoHeader
            ? DriverTests.skipFirstLine(expectedCsvRows)
            : expectedCsvRows;
        expect(string.trim()).toEqual(expectedRows);
    }
    async testUnloadEscapeSymbolOp1(Driver) {
        process.env.CUBEJS_DB_EXPORT_BUCKET_CSV_ESCAPE_SYMBOL = '"';
        const driver = new Driver({});
        const query = `
      SELECT orders.status AS orders__status, sum(orders.amount) AS orders__amount        
      FROM (${DriverTests.QUERY}) AS orders
      GROUP BY 1
      ORDER BY 1
    `;
        const tableName = await this.createUnloadTable(query);
        (0, assert_1.default)(driver.unload);
        const data = await driver.unload(tableName, { maxFileSize: 64 });
        expect(data.exportBucketCsvEscapeSymbol).toBe('"');
        await driver.release();
    }
    async testUnloadEscapeSymbolOp2(Driver) {
        process.env.CUBEJS_DB_EXPORT_BUCKET_CSV_ESCAPE_SYMBOL = '\'';
        const driver = new Driver({});
        const query = `
      SELECT orders.status AS orders__status, sum(orders.amount) AS orders__amount        
      FROM (${DriverTests.QUERY}) AS orders
      GROUP BY 1
      ORDER BY 1
    `;
        const tableName = await this.createUnloadTable(query);
        (0, assert_1.default)(driver.unload);
        const data = await driver.unload(tableName, { maxFileSize: 64 });
        expect(data.exportBucketCsvEscapeSymbol).toBe('\'');
        await driver.release();
    }
    async testUnloadEscapeSymbolOp3(Driver) {
        delete process.env.CUBEJS_DB_EXPORT_BUCKET_CSV_ESCAPE_SYMBOL;
        const driver = new Driver({});
        const query = `
      SELECT orders.status AS orders__status, sum(orders.amount) AS orders__amount        
      FROM (${DriverTests.QUERY}) AS orders
      GROUP BY 1
      ORDER BY 1
    `;
        const tableName = await this.createUnloadTable(query);
        (0, assert_1.default)(driver.unload);
        const data = await driver.unload(tableName, { maxFileSize: 64 });
        expect(data.exportBucketCsvEscapeSymbol).toBeUndefined();
        await driver.release();
    }
    async testUnloadEmpty() {
        const query = `
      SELECT 'new' AS orders__status, 100 AS orders__amount  
      WHERE FALSE  
    `;
        const tableName = await this.createUnloadTable(query);
        (0, assert_1.default)(this.driver.unload);
        const data = await this.driver.unload(tableName, { maxFileSize: 64 });
        expect(data.csvFile.length).toEqual(0);
    }
    async createUnloadTable(query) {
        const versionEntry = {
            table_name: 'test.orders_order_status',
            structure_version: crypto_1.default.randomBytes(10).toString('hex'),
            content_version: crypto_1.default.randomBytes(10).toString('hex'),
            last_updated_at: new Date().getTime(),
            naming_version: 2
        };
        const tableName = query_orchestrator_1.PreAggregations.targetTableName(versionEntry);
        const loadQuery = this.options.wrapLoadQueryWithCtas ? `CREATE TABLE ${tableName} AS ${query}` : query;
        await this.driver.loadPreAggregationIntoTable(tableName, loadQuery, [], {
            newVersionEntry: versionEntry,
            targetTableName: tableName,
        });
        return tableName;
    }
    static skipFirstLine(text) {
        return text.split('\n').slice(1).join('\n');
    }
    static rowsToString(rows) {
        const result = [];
        for (const row of rows) {
            const newRow = {};
            for (const k of Object.keys(row)) {
                newRow[k] = row[k] === null ? null : row[k].toString();
            }
            result.push(newRow);
        }
        return result;
    }
}
exports.DriverTests = DriverTests;
//# sourceMappingURL=DriverTests.js.map